unit OptionUnit;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QButtons, QGrids,Jab;

type
  TfmOptions = class(TForm)
    btOK: TBitBtn;
    btCancel: TBitBtn;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    edWorldDir: TEdit;
    btBrowseWld: TBitBtn;
    GroupBox2: TGroupBox;
    lbConstants: TListBox;
    sgValues: TStringGrid;
    btConstAccept: TBitBtn;

    Procedure ShowConst(Ind : integer);

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btOKClick(Sender: TObject);
    procedure btBrowseWldClick(Sender: TObject);
    procedure lbConstantsClick(Sender: TObject);
    procedure sgValuesKeyPress(Sender: TObject; var Key: Char);
    procedure btConstAcceptClick(Sender: TObject);
  private
    { Private declarations }
  public
    WorldDir : String;
    VectorChanged : Boolean;
  end;

Const
     MAX_OPTION_ITEMS = 1;

var
  fmOptions : TfmOptions;
  Options   : TStringList;
  CurConst  : Integer = -1;

implementation

uses CircleEdUnit;

{$R *.xfm}

procedure TfmOptions.FormCreate(Sender: TObject);
begin
     Options := TStringList.Create;
end;

procedure TfmOptions.FormShow(Sender: TObject);
Var
   i : integer;
begin
     if FileExists('options.dat') then
        Options.LoadFromFile('options.dat');
     If Options.Count >= MAX_OPTION_ITEMS then
     Begin
          WorldDir := Options.Strings[0];
          edWorldDir.Text := WorldDir;
     End;
     VectorChanged := false;
     lbConstants.Clear;
     With fmCircleEd do
     Begin
          if Length(Constants) > 0 then
          for i := 0 to Length(Constants)-1 do
          Begin
               lbConstants.Items.Add(Constants[i].ConName+'  '+Constants[i].ConType)
          End;
     End;
     If lbConstants.Items.Count > 0 then
     Begin
          CurConst := 0;
          lbConstants.Selected[0] := true;
          ShowConst(CurConst);
     End;
     
end;

procedure TfmOptions.btOKClick(Sender: TObject);
Var
   ConstFile : TStringList;
   i,j       : integer;
   CType     : String;
begin
     if MessageDlg('Save Options','Save Options',mtConfirmation,[mbYes,mbNo],0) = mrYes then
     Begin
          WorldDir := edWorldDir.Text;

          Options.Clear;
          Options.Add(WorldDir);

          Options.SaveToFile('options.dat');
          if VectorChanged then
          Begin
               ConstFile := TStringList.Create;
               ConstFile.Add('* Constants for Flag and Bitvector');
               With fmCircleEd do
               Begin
                    For i := 0 to Length(Constants)-1 do
                    Begin
                         CType := Constants[i].ConType;
                         ConstFile.Add('['+Constants[i].ConName+'] ['+CType+']');
                         For j := 0 to Length(Constants[i].Values)-1 do
                         Begin
                              if UpperCase(CType) = 'FLAG' then
                              Begin
                                   ConstFile.Add(IntToStr(Constants[i].Values[j].Flag)+' '+Constants[i].Values[j].Text);
                              End Else
                              if UpperCase(CType) = 'BV' then
                              Begin
                                   ConstFile.Add(Constants[i].Values[j].BV+' '+ IntToStr(Constants[i].Values[j].Flag)+' '+Constants[i].Values[j].Text);
                              End Else
                              if UpperCase(CType) = 'SPEC' then
                              Begin
                                   ConstFile.Add(Constants[i].Values[j].Text);
                              End;
                         End;
                         ConstFile.Add('$');
                         ConstFile.Add('');
                    End;
               End;
               ConstFile.SaveToFile('constant.dat');
          End;
     End;
end;

procedure TfmOptions.btBrowseWldClick(Sender: TObject);
Var
   Direc : String;
begin
     if selectdirectory('Choose Directory','',Direc) then
     Begin
          edWorldDir.Text := Direc;
     End;
end;

Procedure TfmOptions.ShowConst(Ind : integer);
Var
   ValLen,
   i : integer;

Begin
     if (ind < 0) or (ind > Length(fmCircleEd.constants)) then exit;
     With fmCircleEd do
     Begin
          If UpperCase(Constants[ind].ConType) = 'FLAG' then
          Begin
               sgValues.ColCount := 2;
               sgValues.ColWidths[0] := 50;
               sgValues.ColWidths[1] := 180;
               sgValues.Cells[0,0] := 'Value';
               sgValues.Cells[1,0] := 'Description';
               ValLen := Length(Constants[ind].Values);
               if  ValLen > 0 then
               Begin
                    sgValues.RowCount := ValLen+2; //fixed + last
                    sgValues.Rows[ValLen+1].Text := '';
                    for i := 0 to ValLen-1 do
                    Begin
                         sgValues.Cells[0,i+1] := IntToStr(Constants[ind].Values[i].Flag);
                         sgValues.Cells[1,i+1] := Constants[ind].Values[i].Text;
                    End;
               End;
          End Else
          If UpperCase(Constants[ind].ConType) = 'BV' then
          Begin
               sgValues.ColCount := 3;
               sgValues.Cells[0,0] := 'Ascii';
               sgValues.Cells[1,0] := 'Numeric';
               sgValues.Cells[2,0] := 'Description';
               sgValues.ColWidths[0] := 40;
               sgValues.ColWidths[1] := 70;
               sgValues.ColWidths[2] := 120;
               ValLen := Length(Constants[ind].Values);
               if  ValLen > 0 then
               Begin
                    sgValues.RowCount := ValLen+2; //fixed + last
                    sgValues.Rows[ValLen+1].Text := '';
                    for i := 0 to ValLen-1 do
                    Begin
                         sgValues.Cells[0,i+1] := Constants[ind].Values[i].BV;
                         sgValues.Cells[1,i+1] := IntToStr(Constants[ind].Values[i].Flag);
                         sgValues.Cells[2,i+1] := Constants[ind].Values[i].Text;
                    End;
               End;
          End Else
          If UpperCase(Constants[ind].ConType) = 'SPEC' then
          Begin
               sgValues.ColCount := 1;
               sgValues.Cells[0,0] := 'Description';
               sgValues.ColWidths[0] := 100;
               ValLen := Length(Constants[ind].Values);
               if ValLen > 0 then
               Begin
                    sgValues.RowCount := ValLen+2; //fixed + last
                    sgValues.Rows[ValLen+1].Text := '';
                    for i := 0 to ValLen-1 do
                    Begin
                         sgValues.Cells[0,i+1] := Constants[ind].Values[i].Text;
                    End;
               End;
          End Else ShowMessage('Unknown Constant Type');
     End;
End;

procedure TfmOptions.lbConstantsClick(Sender: TObject);
begin
     If lbConstants.ItemIndex > -1 then
     Begin
          CurConst := lbConstants.ItemIndex;
          ShowConst(CurConst);
     End;
end;

procedure TfmOptions.sgValuesKeyPress(Sender: TObject; var Key: Char);
Var
   CType : String;
begin
     if CurConst < 0 then exit;
     Ctype := fmCircleEd.Constants[CurConst].ConType;
     if UpperCase(CType) = 'FLAG' then
     Begin
          if sgValues.Col = 0 then NumOnlyE(Key,false,true);
          if sgValues.Row =  sgValues.RowCount-1 then
          Begin
               if sgValues.RowCount > 2 then
               sgValues.Cells[0,sgValues.RowCount-1] := IntToStr(StrToInt(sgValues.Cells[0,sgValues.RowCount-2])+1);
               sgValues.RowCount := sgValues.RowCount+1;
               sgValues.Rows[sgValues.RowCount-1].Text := '';
          End;
     End Else
     if UpperCase(CType) = 'BV' then
     Begin
          if sgValues.Col = 1 then NumOnlyE(Key,false,true);
          if sgValues.Row =  sgValues.RowCount-1 then
          Begin
               if sgValues.RowCount > 2 then
               sgValues.Cells[1,sgValues.RowCount-1] := IntToStr(StrToInt(sgValues.Cells[1,sgValues.RowCount-2])*2);
               sgValues.RowCount := sgValues.RowCount+1;
               sgValues.Rows[sgValues.RowCount-1].Text := '';
          End;
     End Else
     if UpperCase(CType) = 'SPEC' then
     Begin
          if sgValues.Row =  sgValues.RowCount-1 then
          Begin
               sgValues.RowCount := sgValues.RowCount+1;
               sgValues.Rows[sgValues.RowCount-1].Text := '';
          End;
     End Else ShowMessage('Unknown Constant Type');
end;

procedure TfmOptions.btConstAcceptClick(Sender: TObject);
Var
   ValLen,Code,num,
   i : integer;
begin
     if CurConst < 0 then exit;
     VectorChanged := True;
     With fmCircleEd do
     Begin
          SetLength(Constants[CurConst].Values,0);
          If UpperCase(Constants[CurConst].ConType) = 'FLAG' then
          Begin
               if  sgValues.RowCount > 1 then
               Begin
                    for i := 1 to sgValues.RowCount-1 do
                    Begin
                         ValLen := Length(Constants[CurConst].Values);
                         Val(sgValues.Cells[0,i],num,code);
                         if (code = 0) and (sgValues.Cells[1,i] <> '') then
                         Begin
                              SetLength(Constants[CurConst].Values,ValLen+1);
                              Constants[CurConst].Values[ValLen].Flag := num;
                              Constants[CurConst].Values[ValLen].Text := sgValues.Cells[1,i];
                         End;
                    End;
               End;
          End Else
          If UpperCase(Constants[CurConst].ConType) = 'BV' then
          Begin
               if  sgValues.RowCount > 1 then
               Begin
                    for i := 1 to sgValues.RowCount-1 do
                    Begin
                         ValLen := Length(Constants[CurConst].Values);
                         Val(sgValues.Cells[1,i],num,code);
                         if (code = 0) and
                            (sgValues.Cells[0,i] <> '') and
                            (sgValues.Cells[2,i] <> '') then
                         Begin
                              SetLength(Constants[CurConst].Values,ValLen+1);
                              Constants[CurConst].Values[ValLen].Flag := num;
                              Constants[CurConst].Values[ValLen].BV   := sgValues.Cells[0,i];
                              Constants[CurConst].Values[ValLen].Text := sgValues.Cells[2,i];
                         End;
                    End;
               End;
          End Else
          If UpperCase(Constants[CurConst].ConType) = 'SPEC' then
          Begin
               if  sgValues.RowCount > 1 then
               Begin
                    for i := 1 to sgValues.RowCount-1 do
                    Begin
                         ValLen := Length(Constants[CurConst].Values);
                         if (sgValues.Cells[0,i] <> '') then
                         Begin
                              SetLength(Constants[CurConst].Values,ValLen+1);
                              Constants[CurConst].Values[ValLen].Text := sgValues.Cells[0,i];
                         End;
                    End;
               End;
          End Else ShowMessage('Unknown Constant Type');
     End;
end;

end.
